# -*- coding:utf-8 -*-
import pytest

from blockerbugs.controllers.api import validators
from blockerbugs.controllers.api.errors import ValidationError


class TestValidators(object):
    def test_string(self):
        string_validator = validators.StringValidator()
        valid_string_value = 'abc'
        invalid_type_value = 123
        assert string_validator(valid_string_value)
        with pytest.raises(ValidationError):
            string_validator(invalid_type_value)

    def test_integer_types(self):
        integer_validator = validators.IntegerValidator()
        valid_integer = 123
        invalid_integer1 = 12.3
        invalid_integer2 = '12.3'
        assert integer_validator(valid_integer)
        with pytest.raises(ValidationError):
            integer_validator(invalid_integer1)
        with pytest.raises(ValidationError):
            integer_validator(invalid_integer2)

    def test_integer_min_max_value(self):
        max_int_validator = validators.IntegerValidator(max=100, min=0)
        valid_integer = 50
        invalid_integer = -1
        invalid_integer2 = 101
        max_int_validator(valid_integer)
        with pytest.raises(ValidationError):
            max_int_validator(invalid_integer)
        with pytest.raises(ValidationError):
            max_int_validator(invalid_integer2)

    def test_dict(self):
        dict_validator = validators.DictValidator(
            {'arg1': validators.StringValidator()})
        valid_dict = {'arg1': 'str'}
        invalid_dict = {'arg1': 123}
        invalid_dict2 = {'abc': 'str'}
        assert dict_validator(valid_dict)
        with pytest.raises(ValidationError):
            dict_validator(invalid_dict)
        with pytest.raises(ValidationError):
            dict_validator(invalid_dict2)

    def test_required(self):
        req_string_validator = validators.StringValidator()
        not_req_string_validator = validators.StringValidator(required=False)
        dict_validator = validators.DictValidator({
            'req': req_string_validator,
            'opt': not_req_string_validator,
        })
        valid_dict = {'req': 'abc'}
        valid_dict2 = {'req': 'abc', 'opt': 'def'}
        invalid_dict = {'opt': 'def'}
        assert dict_validator(valid_dict)
        assert dict_validator(valid_dict2)
        with pytest.raises(ValidationError):
            dict_validator(invalid_dict)

    def test_const(self):
        const_validator = validators.ConstValidator({'a': 1})
        valid_data = {'a': 1}
        invalid_data = {'a': '1'}
        assert const_validator(valid_data)
        with pytest.raises(ValidationError):
            const_validator(invalid_data)

    def test_name(self):
        int_validator = validators.IntegerValidator(name='test')
        invalid_data = 'abc'
        try:
            int_validator(invalid_data)
        except ValidationError, e:
            assert 'test' in str(e)
        dict_validator = validators.DictValidator({
            'test_name': validators.ConstValidator('333')
        })
        try:
            dict_validator({})
        except ValidationError, e:
            assert 'test_name' in str(e)
        str_validator = validators.StringValidator()
        try:
            str_validator({})
        except ValidationError, e:
            assert str_validator.default_name in str(e)

    def test_choices(self):
        choices_valid = validators.ChoicesValidator(['abc', 'cd', 123])
        assert choices_valid('abc')
        with pytest.raises(ValidationError):
            choices_valid('foo')
        with pytest.raises(TypeError):
            validators.ChoicesValidator(123)

    def test_type(self):
        type_validator = validators.TypeValidator(int)
        assert type_validator(123)
        with pytest.raises(ValidationError):
            type_validator('str')

        class A(object):
            pass

        class B(A):
            pass

        class D(object):
            pass

        a_validator = validators.TypeValidator(A)
        assert a_validator(A())
        assert a_validator(B())
        with pytest.raises(ValidationError):
            a_validator(D())

    def test_int_list(self):
        int_list_validator = validators.ListValidator(int)
        assert int_list_validator([1, 2, 3, 5555])
        assert int_list_validator([])
        with pytest.raises(ValidationError):
            int_list_validator([1, 2, 3, 'abc'])

    def test_datetime(self):
        datetime_validator = validators.DateTimeValidator()
        date_str = '2007-01-25T12:00:00Z'
        invalid_date_str = '25-01-2007'
        assert datetime_validator(date_str)
        with pytest.raises(ValidationError):
            datetime_validator(invalid_date_str)

    def test_regex_string(self):
        regex_validator = validators.StringRegexValidator('^a', required=False)
        assert regex_validator('abcd')
        with pytest.raises(ValidationError):
            regex_validator('foo')
        assert regex_validator(None)

    def test_url(self):
        url_validator = validators.UrlValidator()
        assert url_validator('http://localhost/foo')
        assert url_validator('https://localhost:5000/foo')
        assert url_validator('https://example.com/foo')
        with pytest.raises(ValidationError):
            url_validator('http://example')



